package net.jaekl.cfb.analyze;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Locale;

import net.jaekl.cfb.xml.messages.MessageCollection;
import net.jaekl.qd.xml.ParseErrorHandler;
import net.jaekl.qd.xml.ParseHandler;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

public class MessageMap {
	static final String MESSAGES = "messages";
	static final String XML = "xml";
	
	MessageCollection m_msgColl;
	File m_findBugsDir;
	
	public MessageMap() {
		m_msgColl = null;
		m_findBugsDir = null;
	}
	
	public MessageCollection getColl() { return m_msgColl; }
	public File getFindBugsDir() { return m_findBugsDir; }

	public void load(File findBugsDir, Locale locale) throws FileNotFoundException, IOException, SAXException
	{
		m_findBugsDir = findBugsDir;
		
		String langName = locale.getLanguage();
		
		File msgXml = new File(findBugsDir.getAbsolutePath() + File.separator + MESSAGES + "_" + langName + "." + XML);
		if (! msgXml.canRead()) {
			msgXml = new File(findBugsDir.getAbsolutePath() + File.separator + MESSAGES + "." + XML);
		}
		
		if (! msgXml.canRead()) {
			throw new FileNotFoundException(msgXml.getAbsolutePath());
		}
		
		parse(new InputSource(new FileInputStream(msgXml)));
		
	}
	
	void parse(InputSource xml) throws FileNotFoundException, IOException, SAXException 
	{
		m_msgColl = new MessageCollection();
		
	    XMLReader reader = XMLReaderFactory.createXMLReader();
	    ParseHandler ph = new ParseHandler(m_msgColl);
	    ParseErrorHandler peh = new ParseErrorHandler();
	    reader.setContentHandler(ph);
	    reader.setErrorHandler(peh);
	    reader.parse(xml);
	}
}
