package net.jaekl.cfb.util;

// Copyright (C) 2015 Christian Jaekl

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class Command {
	public static class Result
	{
		private int m_retCode;
		private String m_stdout;
		private String m_stderr;
		
		Result(int retCode, String stdout, String stderr) {
			m_retCode = retCode;
			m_stdout = stdout;
			m_stderr = stderr;
		}
		
		public int getRetCode() { return m_retCode; }
		public String getStdout() { return m_stdout; }
		public String getStderr() { return m_stderr; }
	}
	
	private static class StreamGobbler extends Thread {
		
		private StringBuilder m_sb;
		private BufferedReader m_br;
		
		public StreamGobbler(InputStream is) {
			m_sb = new StringBuilder();
			m_br = new BufferedReader(new InputStreamReader(is));
		}
		
		@Override
		public void run() {
			String line;
			try {
				line = m_br.readLine();
				while (null != line) {
					m_sb.append(line).append(System.lineSeparator());
					line = m_br.readLine();
				}
			} catch (IOException exc) {
				m_sb.append(Util.stringify(exc));
			}
			finally {
				try {
					m_br.close();
				} catch (IOException exc) {
					m_sb.append(Util.stringify(exc));
				}
			}
		}
		
		public String getOutput() { return m_sb.toString(); }
	}
	
	public Command() 
	{
		// no-op
	}
	
	public Result exec(String cmd) throws IOException 
	{
		int retCode = 0;
		String stdout = "";
		String stderr = "";
		
		Process proc = doRuntimeExec(cmd);
		assert( null != proc );
		
		StreamGobbler stdoutGobbler = new StreamGobbler(proc.getInputStream());
		StreamGobbler stderrGobbler = new StreamGobbler(proc.getErrorStream());
		
		stdoutGobbler.start();
		stderrGobbler.start();
		
		try {
			retCode = proc.waitFor();
			stdoutGobbler.join();
			stderrGobbler.join();
			
			stdout = stdoutGobbler.getOutput();
			stderr = stdoutGobbler.getOutput();
			
		} catch (InterruptedException exc) {
			stderr += Util.stringify(exc);
		}
		
		return new Result(retCode, stdout, stderr);
	}
	
	Process doRuntimeExec(String cmd) throws IOException {
		return Runtime.getRuntime().exec(cmd);
	}
}
